/*
 * Multi-RRT.hpp
 *
 *  Created on: Nov 29, 2012
 *      Author: ddevaurs
 */

#ifndef _MULTI_RRT_HPP_
#define _MULTI_RRT_HPP_

#include "planner/Diffusion/RRT.hpp"


/**
 * The Multi-RRT algorithm.
 */
class MultiRRT: public RRT
{
public:
    //! Constructor.
	MultiRRT(Robot * robot, Graph * graph) : RRT(robot, graph) {
        std::cout << "Construct the Multi-RRT" << std::endl;
    }

    //! Destructor.
    virtual ~MultiRRT() {};

    /**
     * Initialize the Multi-RRT.
     * @return the number of nodes added to the graph
     */
    virtual unsigned init() {
    	this->loadWaypoints();
        return RRT::init();
    }

    /**
     * Main function of the Multi-RRT.
     * @return the number of nodes added to the graph
     */
    unsigned run();

	//! Check whether the Multi-RRT has found a trajectory.
	bool trajFound();

    //! Extract a trajectory going through all the waypoints.
    API::Trajectory * extractTrajectory();

protected:
    //! Load the user-defined waypoints (start, goal, and potential intermediate waypoints).
    void loadWaypoints();

private:
    /**
     * Solve the Traveling Salesman Problem by exhaustive search.
     * @param distances: pairwise distance matrix given as input
     * @param solution: shortest path returned as output
     * @return length of the shortest path
     */
    double TPSbyExhaustiveSearch(std::vector<std::vector<double> > & distances,
			std::vector<unsigned> & solution);

    /**
     * Solve the Traveling Salesman Problem using the Nearest Neighbor heuristic.
     * @param distances: pairwise distance matrix given as input
     * @param solution: shortest path returned as output
     * @return length of the shortest path
     */
    double TPSwithNearestNeighbor(std::vector<std::vector<double> > & distances,
			std::vector<unsigned> & solution);

    /**
     * Solve the Traveling Salesman Problem using the Multi-Fragment heuristic.
     * @param distances: pairwise distance matrix given as input
     * @param solution: shortest path returned as output
     * @return length of the shortest path
     */
	double TPSwithMultiFragments(std::vector<std::vector<double> > & distances,
			std::vector<unsigned> & solution);

protected:
    //! waypoints through which the solution path should go
    std::vector<confPtr_t> waypoints;

    //! nodes containing the waypoints
    std::vector<Node *> waypointNodes;
};

#endif

